<?php defined( '_MAXSITE' ) or die( 'Restricted access' );
/**
 * @package	Atomymaxsite - Maxsite
 * @license	LICENSE.txt
 * @license	GNU General Public License version 2 or later; see LICENSE.txt
 * @since	Version 2.5.0
 */

class MFile{
    
    /**
     * Download file with full path
     * @param type $file
     * @return boolean 
     */
    public function download($file){
        if(file_exists($file)===TRUE)
        {
            header('Content-Description: File Transfer');
            header('Content-Type: application/octet-stream');
            header('Content-Disposition: attachment; filename='.basename($file));
            header('Content-Transfer-Encoding: binary');
            header('Expires: 0');
            header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
            header('Pragma: public');
            header('Content-Length: ' . filesize($file));
            ob_clean();
            flush();
            readfile($file);
            exit;
        }
        else
        {
            return FALSE;
        }
    }
    
    public function formatBytes($bytes, $precision = 2) { 
        $units = array('B', 'KB', 'MB', 'GB', 'TB'); 

        $bytes = max($bytes, 0); 
        $pow = floor(($bytes ? log($bytes) : 0) / log(1024)); 
        $pow = min($pow, count($units) - 1); 

        // Uncomment one of the following alternatives
        // $bytes /= pow(1024, $pow);
        // $bytes /= (1 << (10 * $pow)); 

        return round($bytes, $precision) . ' ' . $units[$pow]; 
    }
    
    public function uploadAttr($file, $path, $pattern, $prefix=FALSE){
        
        $patt = $this->uploadMatch($file['name'], $pattern);
        if($patt > 0 && $file['error']===0)
        {
            $pref = $prefix===FALSE ? TIMESTAMP : $prefix ;
            
            $filename = $pref.'_'.$file['name'];
            if(copy($file['tmp_name'], $path.$filename)===TRUE)
            {
                return $filename;
            }
            else
            {
                return FALSE;
            }
        }
        else
        {
            return FALSE;
        }
    }
    
    /**
     * Check file type from preg_match
     * @param string $filename  name of file
     * @param string $pattern   reg_exp
     * @return type
     */
    public function uploadMatch($filename, $pattern){
        return preg_match('/\.('.$pattern.')$/i', $filename);
    }
    
    public function file_move_error(){
        return FILE_UPLOAD_ERROR;
    }
    
    /**
     * Resize .jpg, .gif, .png
     * 
     * @param $_FILES $file
     * @param int $limit        fix with width or height
     * @param int $width        new width of image
     * @param int $height       new height of image
     * @param string $path      folder to store an image(relative path)
     * @param string $prefix    string before $file['name']
     * @return string
     * 
     * If input $path(info) and $prefix(user_) and $file['name'] is simple.jpg
     * info/user_simple.jpg something like this ;)
     * 
     * limit: 0 is width
     *        1 is height
     *        else is auto resize
     */
    public function resize($file, $limit, $width=NULL, $height=NULL, $path=NULL, $prefix=NULL){
        
        $size = getimagesize($file['tmp_name']);
        
        if($width===NULL)
            $width = $size['0'];
        
        if($height===NULL)
            $height = $size['1'];
        
        if($limit===0)
        {
            $height = floor($width*($size['1']/$size['0']));
            $width = (int)$width;
        }
        else if($limit===1)
        {
            $width = floor($height*($size['0']/$size['1']));
            $height = (int)$height;
        }
        else
        {
            $width = floor($size['1']*($size['0']/$size['1']));
            $height = $size['1'];
        }
        
        // Create a new image instance
        $thumb = imagecreatetruecolor($width, $height);
        
        $info = strrchr(strtolower($file['name']), '.' );
        if($info==='.gif')
        {
            $image = imagecreatefromgif($file['tmp_name']);
        }
        else if($info==='.jpg' OR $info==='.jpge')
        {
            $image = imagecreatefromjpeg($file['tmp_name']);
        }
        else if($info==='.png')
        {
            $image = imagecreatefrompng($file['tmp_name']);
        }
        
        // Resize
        imagecopyresized($thumb, $image, 0, 0, 0, 0, $width, $height, $size[0], $size[1]);
        $filename = $prefix.$file['name'];
        
        if($info==='.gif')
        {
            imagegif($thumb, $path.$filename);
        }
        else if($info==='.jpg' OR $info==='.jpge')
        {
            imagejpeg($thumb, $path.$filename, 100);
        }
        else if($info==='.png')
        {
            imagepng($thumb, $path.$filename);
        }
        imagedestroy($thumb);
        return $filename;
    }
    
    
}